#!/usr/bin/env python

# testcore.py - test of the core module
#
# Copyright 2004, 2006 Floris Bruynooghe
#
# This file is part of Delny.
#
# Delny is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Delny is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Delny; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
#
#
# Authors: Floris Bruynooghe (flub)

"""Test the core module from the delaunay package

This module provides the unittests for the core module of the delaunay
package.
"""


import unittest
import Numeric
import delaunay.core as core
from test import test_support


class TestTriangulation(unittest.TestCase):
    def test_constructor_2d(self):
        # delaunay.core.Triangulation(set): constructs fine? (2D)
        set = [(0,0),(2,1),(1,2),(4,0),(0,4),(4,4)]
        self.assert_(core.Triangulation(set))

    def test_constructor_3d(self):
        # delaunay.core.Triangulation(set): constructs fine? (3D)
        set = [(0,0,1), (4,0,1), (0,4,1), (4,4,0), (2,1,2), (1,2,3)]
        self.assert_(core.Triangulation(set))

    def test_sequence_types(self):
        # delaunay.core.Triangulation(): accepts all sequnce types?
        a = Numeric.array
        ltset = [(0,0),(2,1),(1,2),(4,0),(0,4),(4,4)]
        llset = [[0,0],[2,1],[1,2],[4,0],[0,4],[4,4]]
        tlset = ([0,0],[2,1],[1,2],[4,0],[0,4],[4,4])
        ttset = ((0,0),(2,1),(1,2),(4,0),(0,4),(4,4))
        lnset = [a((0,0)),a((2,1)),a((1,2)),a((4,0)),a((0,4)),a((4,4))]
        nnset = a([(0,0),(2,1),(1,2),(4,0),(0,4),(4,4)])
        all = (ltset, llset, tlset, ttset, lnset, nnset)
        for set in all:
            self.assert_(core.Triangulation(set))

    def test_dim(self):
        # delaunay.core.Triangulation(set, dim=?): constructs fine?
        set = [0,0,2,1,1,2,4,0,0,4,4,4]
        self.assert_(core.Triangulation(set, 2))
        set = [(0),(0),(2),(1),(1),(2),(4),(0),(0),(4),(4),(4)]
        self.assert_(core.Triangulation(set, 2))
        set = [(0,0,2),(1,1,2),(4,0,0),(4,4,4)]
        self.assert_(core.Triangulation(set, 2))
        set = [(0,0,2,1),(1,2,4,0),(0,4,4,4)]
        self.assert_(core.Triangulation(set, 2))


class TestTriangulationMethods(unittest.TestCase):
    def setUp(self):
        self.set = [(0,0),(2,1),(1,2),(4,0),(0,4),(4,4)]
        self.obj = core.Triangulation(self.set)

    def test_get_set(self):
        # delaunay.core.Triangulation.get_set(): returns correct set?
        self.set.sort()
        used_set = self.obj.get_set()
        used_set.sort()
        self.assertEqual(self.set, used_set)

    def test_getneighbours_type(self):
        # delaunay.core.Triangulation.get_neighbours(): correct type?
        # Type is dict?
        self.assert_(isinstance(self.obj.get_neighbours(), dict))
        # Type of keys of dict ok?
        for i in self.obj.get_neighbours().keys():
            self.assert_(isinstance(i, tuple))
            self.assertEqual(len(i), len(self.set[0]))
            for j in i:
                self.assert_(isinstance(j, float))
        # Type of values of dict ok?
        for i in self.obj.get_neighbours().values():
            self.assert_(isinstance(i, list))
            for j in i:
                self.assert_(isinstance(j, tuple))
                self.assertEqual(len(j), len(self.set[0]))
                for k in j:
                    self.assert_(isinstance(k, float))

    def test_getneighbours_points(self):
        # delaunay.core.Triangulation.get_neighbours(): points are from set?
        for i in self.obj.get_neighbours().keys():
            self.assert_(i in self.set)
        for i in self.obj.get_neighbours().values():
            for j in i:
                self.assert_(j in self.set)

    def test_getelements_type(self):
        # delaunay.core.Triangulation.get_elements(): correct type?
        # Type is list?
        self.assert_(isinstance(self.obj.get_elements(), list))
        # List contains lists of tuples  of correct dimension of floats?
        for i in self.obj.get_elements():
            self.assert_(isinstance(i, list))
            for j in i:
                self.assert_(isinstance(j, tuple))
                self.assertEqual(len(j), len(self.set[0]))
                for k in j:
                    self.assert_(isinstance(k, float))

    def test_getelements_points(self):
        # All points returned are from the set?
        for i in self.obj.get_elements():
            for j in i:
                self.assert_(j in self.set)

    def test_get_elements_by_index_type(self):
        # Correct type returned by .get_elements_indices()?
        self.assert_(isinstance(self.obj.get_elements_indices(), list))
        for i in self.obj.get_elements_indices():
            self.assert_(isinstance(i, list))
            for j in i:
                self.assert_(isinstance(j, int))

    def test_updateset(self):
        # delaunay.core.Triangulation.update_set(): modifies triangulation?
        old_elements = self.obj.get_elements()
        old_neighbours = self.obj.get_neighbours()
        new_set = [(0,0),(2,1),(1,2),(5,0),(0,5),(5,5)]
        self.obj.update_set(new_set)
        self.assertEqual(new_set.sort(), self.obj.get_set().sort())
        self.assertNotEqual(old_elements, self.obj.get_elements())
        self.assertNotEqual(old_neighbours, self.obj.get_neighbours())


def test_main():
    test_support.run_unittest(TestTriangulation,
                              TestTriangulationMethods)


if __name__ == '__main__':
    test_main()



# core.py - the main user interface to the Delny package
#
# Copyright 2004-2006 Floris Bruynooghe
#
# This file is part of Delny.
#
# Delny is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Delny is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Delny; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
#
#
# Authors: Floris Bruynooghe (flub)

"""The main user interface to the Delny package

Most users will want to use the Triangulate class to access the
routines provided in the package.
"""

import numpy
import _qhull


class Triangulation:
    """Represents a Delaunay triangulation of a set of points

    All sequences used in this class can be either Python builtin
    sequences or numpy sequences.  Results are returned as Python
    builtin sequences however.
    """
    # Data attributes:
    #   self.neighbours
    #   self.facets
    #   self.indices
    def __init__(self, inputset, dim=None):
        """Creates the Delaunay triangulation

        The `set' can be a 2 dimensional sequence with the last
        dimension being the coordinates of every point.  In other
        words: a sequence of points where each point is a sequence
        itself.

        Alternatively the set is a sequence objet of any dimension.
        In this case the `dim' argument should be given and indicate
        the number of dimensions.  In this case the sequence will be
        flattened (made 1-dimensional) and the first point will be
        represented by the `dim' first elements of it and so on.
        """
        if(dim != None):
            self.set = numpy.array(inputset)
            self.set.shape = (-1, dim)
        else:
            self.set = inputset
        self.neighbours, self.facets, self.indices = _qhull.delny(self.set)

    def get_set(self):
        """Returns the set as it is being used by this class

        This could be any sequence object, however if the `dim'
        argument was not passed along to the constructor of the object
        you can be sure this is the same sequence object as you passed
        to the constructor.
        """
        return self.set

    def get_neighbours(self):
        """Returns the neighbours of each point in the set

        This is a dictionnary with the points of the sets as key and a
        list of it's nearest neighbours as value.  Every neighbour is
        a tuple with <dimension> floats.
        """
        return self.neighbours

    def get_elements(self):
        """Returns the elements of the Delaunay triangulation

        This is a list of elements where every element is a list of
        nodes and every node is a tuple of <dimesnion> floats.  An
        element is a triangle in 2D and a tetraheron in 3D.
        """
        return self.facets

    def get_elements_indices(self):
        """Returns the elements of the Delaunay triangulation

        This is a list of elements where every element is a list of
        node indices corresponding to the point index given in the inputset.
        An element is a triangle in 2D and a tetraheron in 3D.
        """
        return self.indices


    def update_set(self, newset):
        """Recalculate the neighbours with a new input set

        This has the same effect as creating a new instance but
        without doing so.
        """
        #FIXME: should this be renamed to set_set()?
        #FIXME: should this also take the `dim' argument?
        self.__init__(newset)



#!/usr/bin/env python

# test_qhull.py - test of _qhull module
#
# Copyright 2004, 2006 Floris Bruynooghe
#
# This file is part of Delny.
#
# Delny is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Delny is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Delny; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
#
#
# Author: Floris Bruynooghe (flub)

"""Test the _qhull module from the delaunay package

This module provides the unittests for the _qhull module of the delaunay
package.
"""


import unittest

import Numeric
import RandomArray

import delaunay._qhull

from test import test_support


class TestQhullFunctions(unittest.TestCase):
    """This tests the types and structures returned."""
    def setUp(self):
        # Prepare a set of points and their neighbours.
        self.set = [(0,0),(2,1),(1,2),(4,0),(0,4),(4,4)]
        result = delaunay._qhull.delny(self.set)
        self.neighbours = result[0]
        self.elements = result[1]
        self.elements_by_index = result[2]

    def test_nb_dict(self):
        # delaunay._qhull.delny: neighbours are returned as a dict?
        self.assert_(isinstance(self.neighbours, dict))

    def test_nb_keys(self):
        # delaunay._qhull.delny: neighbours keys are of correct type?
        for i in self.neighbours.keys():
            self.assert_(isinstance(i, tuple))
            self.assertEqual(len(i), len(self.set[0]))
            for j in i:
                self.assert_(isinstance(j, float))

    def test_nb_values(self):
        # delaunay._qhull.delny: neighbours values are of correct type?
        for i in self.neighbours.values():
            self.assert_(isinstance(i, list))
            for j in i:
                self.assert_(isinstance(j, tuple))
                self.assertEqual(len(j), len(self.set[0]))
                for k in j:
                    self.assert_(isinstance(k, float))

    def test_nb_points(self):
        # delaunay._qhull.delny: all points in neighbours are from the set?
        for i in self.neighbours.keys():
            self.assert_(i in self.set)
        for i in self.neighbours.values():
            for j in i:
                self.assert_(j in self.set)
        
    def test_elem_list(self):
        # delaunay._qhull.delny: returned elements are in a list?
        self.assert_(isinstance(self.elements, list))

    def test_elem(self):
        # delaunay._qhull.delny: returned elements have correct type?
        for i in self.elements:
            self.assert_(isinstance(i, list))
            for j in i:
                self.assert_(isinstance(j, tuple))
                self.assertEqual(len(j), len(self.set[0]))
                for k in j:
                    self.assert_(isinstance(k, float))

    def test_elem_points(self):
        # delaunay._qhull.delny: all points from elements are in the set?
        for i in self.elements:
            for j in i:
                self.assert_(j in self.set)

    def test_elem_index(self):
        # Type of facet_list is correct?
        self.assert_(isinstance(self.elements_by_index, list))
        for i in self.elements_by_index:
            self.assert_(isinstance(i, list))
            for j in i:
                self.assert_(isinstance(j, int))


class TestPredictedOutputs2D(unittest.TestCase):
    """Values used in this class come from the qhull binary"""

    def setUp(self):
        self.set = [(0,0),(2,1),(1,2),(4,0),(0,4),(4,4)]
        self.nb, self.el, self.el_i = delaunay._qhull.delny(self.set)
    
    def test_neighbours(self):
        # delaunay._qhull.delny: neighbours are as predicted? (2D)
        nb = {(0,0): [(2,1), (4,0), (1,2), (0,4)],
              (0,4): [(1,2), (0,0), (4,4)],
              (1,2): [(0,4), (0,0), (2,1), (4,4)],
              (2,1): [(4,0), (0,0), (4,4), (1,2)],
              (4,0): [(2,1), (0,0), (4,4)],
              (4,4): [(2,1), (4,0), (1,2), (0,4)]}
        # sort the lists for each key
        for l1, l2 in zip(nb.values(), self.nb.values()):
            l1.sort()
            l2.sort()
        self.assertEqual(self.nb, nb)
        
    def test_elements(self):
        # delaunay._qhull.delny: elements are as predicted? (2D)
        el = [[(2,1), (4,0), (0,0)],
              [(4,4), (2,1), (4,0)],
              [(1,2), (0,4), (0,0)],
              [(1,2), (2,1), (0,0)],
              [(1,2), (4,4), (0,4)],
              [(1,2), (4,4), (2,1)]]
        for l in el:
            l.sort()
        for l in self.el:
            l.sort()
        el.sort()
        self.el.sort()
        self.assertEqual(self.el, el)

    def test_elements_by_index(self):
        # Elements are as predicted?  (2d)
        el_i = [[1, 3, 0], [5, 1, 3], [2, 4, 0],
                [2, 1, 0], [2, 5, 4], [2, 5, 1]]
        for l in el_i:
            l.sort()
        for l in self.el_i:
            l.sort()
        el_i.sort()
        self.el_i.sort()
        self.assertEqual(self.el_i, el_i)

    def test_square(self):
        # delaunay._qhull.delny: 2D square triangulates?
        set = [(0,0),(1,0),(0,1),(1,1)]
        self.assert_(delaunay._qhull.delny(set))

    def test_three_by_three_square(self):
        # 2D 3x3 square triangulates correctly?
        # XXX This fails currently
        vertices = [(-1,-1),(-1,0),(-1,1),
                    ( 0,-1),( 0,0),( 0,1),
                    ( 1,-1),( 1,0),( 1,1)]
        expected = [[1,3,4],[3,1,0],[3,7,4],[7,3,6],
                    [5,1,4],[1,5,2],[7,5,4],[5,7,8]]
        elements_i = delaunay._qhull.delny(vertices)[2]
        for f in expected:
            f.sort()
        expected.sort()
        for f in elements_i:
            f.sort()
        elements_i.sort()
        self.assertEqual(elements_i, expected)


class TestPredictedOutputs3D(unittest.TestCase):
    """Values used in this class come from the qhull binary"""

    def setUp(self):
        self.set = [(0,0,1), (4,0,1), (0,4,1), (4,4,0), (2,1,2), (1,2,3)]
        self.nb, self.el, self.el_i = delaunay._qhull.delny(self.set)
    
    def test_neigbours(self):
        # delaunay._qhull.delny: neighbours are as predicted? (3D)
        nb = {(0,0,1): [(4,0,1), (0,4,1), (4,4,0), (2,1,2), (1,2,3)],
              (4,0,1): [(0,0,1), (4,4,0), (2,1,2), (1,2,3)],
              (0,4,1): [(0,0,1), (4,4,0), (2,1,2), (1,2,3)],
              (4,4,0): [(0,0,1), (4,0,1), (0,4,1), (2,1,2), (1,2,3)],
              (2,1,2): [(0,0,1), (4,0,1), (0,4,1), (4,4,0), (1,2,3)],
              (1,2,3): [(0,0,1), (4,0,1), (0,4,1), (4,4,0), (2,1,2)]}
        for l in nb.values():
            l.sort()
        for l in self.nb.values():
            l.sort()
        self.assertEqual(self.nb, nb)

    def test_elements(self):
        # delaunay._qhull.delny: elements are as predicted? (3D)
        el = [[(2,1,2), (4,4,0), (4,0,1), (0,0,1)],
              [(2,1,2), (4,4,0), (1,2,3), (4,0,1)],
              [(2,1,2), (1,2,3), (0,4,1), (0,0,1)],
              [(4,4,0), (2,1,2), (0,4,1), (0,0,1)],
              [(4,4,0), (2,1,2), (1,2,3), (0,4,1)]]
        for l in el:
            l.sort()
        for l in self.el:
            l.sort()
        el.sort()
        self.el.sort()
        self.assertEqual(self.el, el)

    def test_elements_by_index(self):
        # Elements by index are as predicted?
        el_i = [[4, 3, 1, 0], [4, 3, 5, 1],
                [4, 5, 2, 0], [3, 4, 2, 0], [3, 4, 5, 2]]
        for l in el_i:
            l.sort()
        for l in self.el_i:
            l.sort()
        el_i.sort()
        self.el_i.sort()
        self.assertEqual(self.el_i, el_i)


class EatingOfNodes(unittest.TestCase):
    def setUp(self):
        self.number = 50
        X = RandomArray.random(self.number)
        Y = RandomArray.random(self.number)
        Z = RandomArray.random(self.number)
        co = Numeric.array([X, Y, Z])
        self.points = []
        for i in range(len(co[0])):
            self.points.append(tuple(co[:,i].tolist()))

    def test_no_nodes(self):
        #_qhull.delny(set) does not eat nodes?
        # Do this many times as it does not always happen
        for i in range(self.number):
            neighbours, elements, elements_by_index \
                        = delaunay._qhull.delny(self.points)
            self.assertEqual(self.number, len(neighbours))


def test_main():
    test_support.run_unittest(TestQhullFunctions,
                              TestPredictedOutputs2D,
                              TestPredictedOutputs3D,
                              EatingOfNodes)


if __name__ == '__main__':
    test_main()



#!/usr/bin/env python

# setup.py - build and install the Delny package
#
# Copyright 2004, 2009 Floris Bruynooghe
#
# This file is part of Delny.
#
# Delny is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Delny is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Delny; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.

"""Delaunay triangulation

Python package to create N-dimensional Delaunay triangulations.  It
uses the libqhull library from the Qhull_ project for the
triangulation so a proven algorithm is used.

.. _Qhull: http://www.qhull.org
"""

classifiers = """\
Development Status :: 3 - Alpha
Intended Audience :: Developers
Intended Audience :: Science/Research
License :: DFSG approved
License :: OSI Approved :: GNU General Public License (GPL)
Natural Language :: English
Operating System :: OS Independent
Programming Language :: C
Programming Language :: Python
Topic :: Scientific/Engineering
"""


from distutils.core import setup, Extension

import numpy


# patch distutils if it can't cope with the "classifiers" or
# "download_url" keywords
import sys
if sys.version < '2.2.3':
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None


module_qhull=Extension("delaunay._qhull",
                       sources=["delaunay/_qhullmodule.c"],
                       libraries=["qhull"],
                       include_dirs=[numpy.get_include()])

doclines = __doc__.split("\n")

setup(name="Delny",
      version="0.4.1",
      author="Floris Bruynooghe",
      author_email="floris.bruynooghe@gmail.com",
      maintainer="Floris Bruynooghe",
      maintainer_email="floris.bruynooghe@gmail.com",
      url="http://flub.stuffwillmade.org/delny",
      license="GNU General Pucblic License (GPL)",
      description=doclines[0],
      long_description="\n".join(doclines[2:]),
      keywords=["delaunay", "qhull", "geometry", "triangulation"],
      platforms=["any"],
      download_url="http://pypi.python.org/pypi/Delny",
      classifiers=filter(None, classifiers.split("\n")),
      packages=["delaunay"],
      ext_modules=[module_qhull])



#!/usr/bin/env python

# all.py - run all test modules in this directory.
#
# Copyright 2006 Floris Bruynooghe
#
# This file is part of Delny.
#
# Delny is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Delny is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Delny; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
#
#
# Authors: Floris Bruynooghe (flub)


import os
import os.path


testdir = os.path.abspath(os.path.dirname(__file__))
files = [os.path.join(testdir, f) for f in os.listdir(testdir)]
test_modules = []
for f in filter(os.path.isfile, files):
    if f[-3:] != ".py":
        continue
    f = os.path.basename(f)
    if f.startswith("test_"):
        test_modules.append(f[:-3])

for module in map(__import__, test_modules):
    print "\nRunning tests from %s:" % module.__name__
    print "="*(20+len(module.__name__)), "\n"
    module.test_main()



# __init__.py - initialisation script of the Delny package
#
# Copyright 2004 Floris Bruynooghe
#
# This file is part of Delny.
#
# Delny is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Delny is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Delny; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
#
#
# Authors: Floris Bruynooghe (flub)

"""Delaunay triangulation.

This package provides a python interface to the Delaunay triangulation
provided by the qhull software package using the C library libqhull.
It is possible to make a triangulations of any dimension you want.

The Delaunay module imports the core module directy in it's namespace
so you can for example use `Delaunay.Triangulation()' after a `import
Delaunay'.  So see the Delaunay.core documentation for the use of the
module.

Whenever a "sequence object" is mentioned in this package all the
builtin Python sequences are accepted as well as an array from the
Numeric module.
"""

from core import *

__all__ = ["core", "_qhull"]
